@@ -35,10 +35,12 @@
 /* Primitive types */
 /** Unicode character */
 typedef unsigned int unichar_t;
-/** Character property */
+/** Character property
+ * @ingroup linebreak */
 typedef unsigned char propval_t;
 
-/** Unicode string */
+/** Unicode string
+ * @ingroup gcstring,linebreak,linebreak_break */
 typedef struct {
     /** Sequence of Unicode character.
      * Note that NUL character (U+0000) may be contained.
@@ -48,7 +50,9 @@ typedef struct {
     size_t len;
 } unistr_t;
 
-/** Grapheme cluster */
+/** Grapheme cluster
+ * @ingroup gcstring
+ */
 typedef struct {
     /** Offset of Unicode string. */
     size_t idx;
@@ -64,7 +68,8 @@ typedef struct {
     unsigned char flag;
 } gcchar_t;
 
-/** Property map entry */
+/** Property map entry
+ * @ingroup linebreak */
 typedef struct {
     /** Beginning of UCS range. */
     unichar_t beg;
@@ -80,7 +85,8 @@ typedef struct {
     propval_t scr;
 } mapent_t;
 
-/** Grapheme cluster string. */
+/** Grapheme cluster string.
+ * @ingroup gcstring,linebreak,linebreak_break */
 typedef struct {
     /** Sequence of Unicode characters.
      * Note that NUL character (U+0000) may be contained.
@@ -99,11 +105,11 @@ typedef struct {
     void *lbobj;
 } gcstring_t;
 
-/** LineBreak object */
+/** LineBreak object.
+ * @ingroup linebreak */
 typedef struct {
-    /***
-     * private members
-     */
+    /** @name private members
+     *@{*/
     /** reference count */
     unsigned long int refcount;
     /** state */
@@ -116,10 +122,10 @@ typedef struct {
     double bufcols;
     /** unread input */
     unistr_t unread;
+    /*@}*/
 
-    /***
-     * public members
-     */
+    /** @name public members
+     *@{*/
     /** Maximum number of Unicode characters each line may contain. */
     size_t charmax;
     /** Maximum number of columns. */
@@ -137,7 +143,7 @@ typedef struct {
     void *format_data;
     void *sizing_data;
     void *urgent_data;
-    void *user_data;
+    void *user_data; /** obsoleted. */
     /** User-defined private data. */
     void *stash;
     /** Format callback function.  See utils.c. */
@@ -146,12 +152,12 @@ typedef struct {
     double (*sizing_func)();
     /** Urgent breaking callback function.  See utils.c. */
     gcstring_t *(*urgent_func)();
-    /** Preprocessing callback function.  See utils.c. */
+    /** Preprocessing callback function.  See utils.c. - obsoleted. */
     gcstring_t *(*user_func)();
     /** Reference Count function.
      * This may be called with 3 arguments: ref_func(data, type, action).
      * data is a (pointer to) external object assinged to stash, format_data,
-     * sizing_data, urgent_data or user_data members.  type is type of object.
+     * sizing_data, urgent_data or prep_data members.  type is type of object.
      * according to action being negative or positive, this function should
      * decrement or increment reference count of object, respectively.
      */
@@ -160,6 +166,15 @@ typedef struct {
      * may be a value of errno defined in <errno.h> or LINEBREAK_ELONG below.
      */
     int errnum;
+    /*@}*/
+
+    /** @name public members addendum on release 2011.1.
+     *@{*/
+    /** Array of preprocessing callback functions.  See utils.c. */
+    gcstring_t *(**prep_func)();    
+    /** Data argument of each preprocessing callback functions. See utils.c. */
+    void **prep_data;
+    /*@}*/
 } linebreak_t;
 
 /***
@@ -169,21 +184,26 @@ typedef struct {
 /** General: Unknown property value. */
 #define PROP_UNKNOWN ((propval_t)~0)
 
-/** gcchar_t: standard flag values. */
-#define LINEBREAK_FLAG_BREAK_BEFORE (2)
+/** @ingroup gcstring
+ * standard flag values. */
 #define LINEBREAK_FLAG_PROHIBIT_BEFORE (1)
+#define LINEBREAK_FLAG_ALLOW_BEFORE (2)
+#define LINEBREAK_FLAG_BREAK_BEFORE LINEBREAK_FLAG_ALLOW_BEFORE
 
-/** linebreak_t: default of charmax member. */
+/** @ingroup linebreak
+ * default of charmax member. */
 #define LINEBREAK_DEFAULT_CHARMAX (998)
 
-/** linebreak_t: bitwise options. */
+/** @ingroup linebreak
+ * bitwise options. */
 #define LINEBREAK_OPTION_EASTASIAN_CONTEXT (1)
 #define LINEBREAK_OPTION_HANGUL_AS_AL (2)
 #define LINEBREAK_OPTION_LEGACY_CM (4)
 #define LINEBREAK_OPTION_BREAK_INDENT (8)
 #define LINEBREAK_OPTION_COMPLEX_BREAKING (16)
 
-/** linebreak_t: state argument for format callback. */
+/** @ingroup linebreak
+ * state argument for format callback. */
 typedef enum {
     LINEBREAK_STATE_NONE = 0,
     LINEBREAK_STATE_SOT, LINEBREAK_STATE_SOP, LINEBREAK_STATE_SOL,
@@ -191,25 +211,30 @@ typedef enum {
     LINEBREAK_STATE_EOL, LINEBREAK_STATE_EOP, LINEBREAK_STATE_EOT,
     LINEBREAK_STATE_MAX
 } linebreak_state_t;
-/** internal states. */
+/** @ingroup linebreak
+ * internal states. */
 #define LINEBREAK_STATE_SOT_FORMAT (-LINEBREAK_STATE_SOT)
 #define LINEBREAK_STATE_SOP_FORMAT (-LINEBREAK_STATE_SOP)
 #define LINEBREAK_STATE_SOL_FORMAT (-LINEBREAK_STATE_SOL)
 
-/** linebreak_t: type argument of ref_func callback. */
+/** @ingroup linebreak
+ * type argument of ref_func callback. */
 #define LINEBREAK_REF_STASH (0)
 #define LINEBREAK_REF_FORMAT (1)
 #define LINEBREAK_REF_SIZING (2)
 #define LINEBREAK_REF_URGENT (3)
 #define LINEBREAK_REF_USER (4)
+#define LINEBREAK_REF_PREP (5)
 
-/** Line breaking action. */
+/** @ingroup linebreak
+ * Line breaking action. */
 #define LINEBREAK_ACTION_MANDATORY (4)
 #define LINEBREAK_ACTION_DIRECT (3)
 #define LINEBREAK_ACTION_INDIRECT (2)
 #define LINEBREAK_ACTION_PROHIBITED (1)
 
-/** linebreak_t: errnum value for exsessive line. */
+/** @ingroup linebreak
+ * special errnum value. */
 #define LINEBREAK_ELONG (-2)
 #define LINEBREAK_EEXTN (-3)
 
@@ -218,6 +243,10 @@ typedef enum {
  *** Public functions, global variables and macros.
  ***/
 
+extern void linebreak_charprop(linebreak_t *, unichar_t,
+			       propval_t *, propval_t *, propval_t *,
+			       propval_t *);
+
 extern gcstring_t *gcstring_new(unistr_t *, linebreak_t *);
 extern gcstring_t *gcstring_newcopy(unistr_t *, linebreak_t *);
 extern gcstring_t *gcstring_copy(gcstring_t *);
@@ -245,6 +274,7 @@ extern void linebreak_destroy(linebreak_t *);
 extern void linebreak_set_newline(linebreak_t *, unistr_t *);
 extern void linebreak_set_stash(linebreak_t *, void *);
 extern void linebreak_set_format(linebreak_t *, gcstring_t *(*)(), void *);
+extern void linebreak_add_prep(linebreak_t *, gcstring_t *(*)(), void *);
 extern void linebreak_set_sizing(linebreak_t *, double (*)(), void *);
 extern void linebreak_set_urgent(linebreak_t *, gcstring_t *(*)(), void *);
 extern void linebreak_set_user(linebreak_t *, gcstring_t *(*)(), void *);
@@ -271,6 +301,8 @@ extern gcstring_t *linebreak_format_NEWLINE(linebreak_t *, linebreak_state_t,
 					    gcstring_t *);
 extern gcstring_t *linebreak_format_TRIM(linebreak_t *, linebreak_state_t,
 					 gcstring_t *);
+extern gcstring_t *linebreak_prep_URIBREAK(linebreak_t *, void *, unistr_t *,
+					   unistr_t *);
 extern double linebreak_sizing_UAX11(linebreak_t *, double, gcstring_t *,
 				     gcstring_t *, gcstring_t *);
 extern gcstring_t *linebreak_urgent_ABORT(linebreak_t *, gcstring_t *);
@@ -279,5 +311,7 @@ extern gcstring_t *linebreak_urgent_FORCE(linebreak_t *, gcstring_t *);
 #define _SOMBOK_H_
 #endif /* _SOMBOK_H_ */
 
-// #include "src/mymalloc.h"
+#ifdef MALLOC_DEBUG
+#include "src/mymalloc.h"
+#endif /* MALLOC_DEBUG */
 
